# coding = utf-8
#!/usr/bin/env python

# ------------------------------ Subscriber --------------------------------
# Description: Low-level Subscriber Class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import zmq
import simplejson
import sys
import nep

try:
    import nanomsg
except ImportError:
    pass

class subscriber:
    """ Subscriber class used for inter-process comunication between nodes. Supports ZeroMQ, nanomsg and ROS publishers. 

        Parameters
        ----------

        topic : string 
            topic name to publish the messages

        node : string or ROS node
            Name of the node or ROS 2.0 instance

        msg_type : string
            Message type: "string", "json" for ZMQ and nanomsg, some ROS geometry messages are also supported 

        conf : dictionary
            Configuration of the publisher.

        callback : function
            Callback function (Only ROS).

        debug: bool
            If True some additional information of the subscriber is shown

    """
    def __init__(self, topic, node = "default", msg_type = "dict", conf =  {'transport': "ZMQ", 'network': "broker", 'mode':"many2many"}, callback = "", debug = False):

        self.callback = callback
        self.topic = topic
        self.node = node
        self.conf = conf
        self.network = self.conf['network']
        self.transport =  self.conf['transport']
        self.msg_type = msg_type
        self.ip = ""
        self.port = ""
        self.debug = debug

        if self.transport ==  "ROS":
            print ("SUB: " + topic + " using ROS 1.0")                                        #Use ROS
            self.__create_ROS_subscriber()
        elif self.transport ==  "ROS2":                                       #Use ROS 2.0
            print ("SUB: " + topic + " using ROS 2.0")
            self.__create_ROS2_subscriber()
        elif self.transport ==  "ZMQ":                                      #Use ZeroMQ
            if self.topic != "/nep_node":
                print ("SUB: " + topic + ", using ZMQ " + self.network)
            self.mode = "many2many" #many2many is the default value
            if "mode" in self.conf:
                self.mode = self.conf['mode']
            self.__create_ZMQ_subscriber()

        if self.transport ==  "NN" or self.transport ==  "nanomsg":          #Use nanomsg
            if self.topic != "/nep_node": 
                print ("SUB: " + topic + " using nanomsg " + self.network)
            self.mode = "many2many" #many2many is the default value #TODO: XPUB XSUB nn_device
            if "mode" in self.conf:
                self.mode = self.conf['mode']
            self.__create_NN_subscriber()
        

    def __network_selection(self):
        """ Get IP and port of this socket

        Returns
        ----------

        success : bool
            Only if True socket can be connected

        port : string
            Port used to connect the socket

        ip : string
            IP used to connect the socket
        """
        success = False
        ip = "127.0.0.1"
        port = "7000"
        self.pid = os.getpid()
        if self.network == "direct":
            # Set the port and ip selected by the user
            port = self.conf["port"]
            ip = self.conf['ip']
            success =  True
        elif self.network == "broker":
            if self.topic != "/nep_node":
                print("SUB: " + self.topic + " waiting NEP master ...")
            # Register the topic in the NEP Master and get the port and ip
            success, port, ip  = nep.masterRegister(self.node, self.topic, master_ip = '127.0.0.1', master_port = 7000, socket = "subscriber", mode = self.mode, pid = self.pid)
            if self.topic != "/nep_node":
                print("SUB: " + self.topic + " socket ready")
        return success, port, ip

    # ---------------------------------- ROS ----------------------------------------
    def __create_ROS_subscriber(self):
        """Function used to create a ROS subscriber"""

        import rospy

        from std_msgs.msg import String
        rospy.Subscriber(self.topic, String, self.__process_ROS_callback)
    
        rospy.init_node(self.node, anonymous = True)



    # ---------------------------------- ROS 2.0 ----------------------------------------
    def __create_ROS2_subscriber(self):
        """Function used to create a ROS subscriber"""

        from std_msgs.msg import String
        self.ros_sub = self.node.create_subscription(String, self.topic, self.__process_ROS_callback)


    def __process_ROS_callback(self,data):
        if self.msg_type == "string":
            message = data.data
            self.callback(message)
        elif self.msg_type == "json":
            msg = data.data
            message = self.__loads(msg)
            self.callback(message)
        


    # ---------------------------------- ZMQ ----------------------------------------
    def __create_ZMQ_subscriber(self):
        """Function used to create a ZeroMQ subscriber"""
        success, self.port, self.ip = self.__network_selection()
        if success:
            try:        
                # ZeroMQ Context
                self.context = zmq.Context()
                # Define the type of context, in this case a subcriber
                self.sock = self.context.socket(zmq.SUB)
                # Define subscription and the messages with prefix to accept.
                # setsockopt obtain the data which message starts with the second argument
                # Then we obtain data from the topic with that starts with self.topic value
                if sys.version_info[0] == 2:
                    self.sock.setsockopt(zmq.SUBSCRIBE, self.topic)
                    self.sock.setsockopt(zmq.CONFLATE, 1) # Only keeps last mesange in queue
                else:
                    self.sock.setsockopt_string(zmq.SUBSCRIBE, self.topic)
                
                self.__connect_ZMQ_socket()
            except:
                print ("NEP ERROR: socket already in use")
        else:
            print("NEP ERROR: Socket unable to be connected")


    def __connect_ZMQ_socket(self):
        """ Connect ZMQ socket in base it configuration
        """

        if self.mode == "many2one":
            endpoint = "tcp://" + self.ip + ":" + str(self.port)
            self.sock.bind(endpoint)
            if self.debug or self.network == "direct":
                if not self.topic == "/nep_node":
                     print("SUB: " + self.topic + " endpoint " + endpoint +  " bind")
        elif self.mode == "one2many":

            endpoint = "tcp://" + self.ip + ":" + str(self.port)
            self.sock.connect(endpoint)
            if self.debug or self.network == "direct":
                if not self.topic == "/nep_node":
                    print("SUB: " + self.topic + " endpoint " + endpoint +  " connect")

        elif self.mode == "many2many":
            endpoint = "tcp://" +  self.ip + ":" + str( self.port+1)
            self.sock.connect(endpoint)
            if self.debug or self.network == "direct":
                if not self.topic == "/nep_node":
                    print("SUB: " + self.topic + " endpoint " + endpoint +  " connect")

        else:
            msg = "NEP ERROR: mode value " +  self.mode + "is not valid"
            raise ValueError(msg)

    def close_ZMQ_subscriber(self):
        """ This function closes the socket"""
        self.sock.close()
        self.context.destroy()
        time.sleep(1)

    # ---------------------------------- Nanomsg ----------------------------------------
    def __create_NN_subscriber(self):
        """Function used to create a NN publisher"""
        try:
            import nanomsg
            self.NN_installed = True
        except ImportError:
            print ("NEP WARNING: Nanomsg not installed")
            self.NN_installed = False

        if self.NN_installed == False:
            msg = "Unable to use surveyor pattern due that Nanomsg is not installed "
            raise ValueError(msg)
        else:
            success, self.port, self.ip = self.__network_selection()
            if success: 
                try:  
                    self.sock = nanomsg.Socket(nanomsg.SUB)
                    self.sock.set_string_option(nanomsg.SUB, nanomsg.SUB_SUBSCRIBE, self.topic)
                    #TODO: zmq.CONFLATE in nanomsg # Only keeps last mesange in queue
                    self.__connect_NN_socket()
                except Exception as e:
                    exc_type, exc_obj, exc_tb = sys.exc_info()
                    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                    print(exc_type, fname, exc_tb.tb_lineno)


    def __connect_NN_socket(self):
        """Function used to create a Nanomsg publisher"""

        if self.mode == "many2one":
            if self.ip == '127.0.0.1':
                 self.ip = "*"
            endpoint = "tcp://" + self.ip + ":" + str(self.port)
            self.sock.bind(endpoint)
            #if self.debug or self.network == "direct":
                #print ("subscriber " + endpoint +  " bind")
        elif self.mode == "one2many":
            if self.ip == '127.0.0.1':
                self.ip = "localhost"
            endpoint = "tcp://" + self.ip + ":" + str(self.port)
            self.sock.connect(endpoint)
            #if self.debug or self.network == "direct":
                #print ("subscriber " + endpoint +  " connect")
        elif  self.mode == "many2many":
            if self.ip == '127.0.0.1':
                self.ip = "localhost"
            endpoint = "tcp://" + self.ip + ":" + str(self.port+1)
            self.sock.connect(endpoint)
            #if self.debug or self.network == "direct":
                #print ("subcriber " + endpoint +  " connect")

        else:
            msg = "NEP ERROR: mode value " +  self.mode + "is not valid"
            raise ValueError(msg)

    def __dumps(self,o, **kwargs):
        """Serialize object to JSON bytes (utf-8). See jsonapi.jsonmod.dumps for details on kwargs.

        Returns
        -------
        message : string
            Encoded string 

        """

        if 'separators' not in kwargs:
            kwargs['separators'] = (',', ':')
        
        s = simplejson.dumps(o, **kwargs)
        

        if sys.version_info[0] == 2: #Python 2
            if isinstance(s, unicode):
                s = s.encode('utf8')
        return s


    def __loads(self, s, **kwargs):
        """Load object from JSON bytes (utf-8). See jsonapi.jsonmod.loads for details on kwargs.
        """
        if sys.version_info[0] == 2:
            if str is unicode and isinstance(s, bytes):
                s = s.decode('utf8')
        return simplejson.loads(s, **kwargs)


    def __deserialization(self, info):
        """ Separate the topic and the json message from the data received from the ZeroMQ socket. If the deserialization was successful and the message as a python dictionary
            
            Parameters
            ----------
            info : string
                topic + message received by the ZQM socket 

            Returns
            -------
            msg : string
                String message as a python dictionary
        """
        try:
            json0 = info.find('{')
            topic = info[0:json0].strip()
            msg = self.__loads(info[json0:])
            success = True
        except:
            msg = ""
            success = False
        return success, msg

    def listen(self,block_mode = False):
        """ Listen data from publishers
            
            Returns
            -------
            message : string or dict
                Return message, must be the same type that msg_type
        """

        if self.msg_type == "string":
            return self.listen_string(block_mode)
        elif self.msg_type == "json":
            return self.listen_json(block_mode)
        else:
            msg = "NEP ERROR: msg_type selected '" + str(self.msg_type) + "' non compatible"
            raise ValueError(msg)



    def listen_string(self, block_mode = False):
        """ Function used to read string data from the sokect. The operation is by default in non blocking mode

            Parameters
            ----------
            block_mode : bool
                If True, the socket will set in blocking mode, otherwise the socket will be in non blocking mode
                
            Returns
            -------
            success : bool
                If True the information was obtained inside the timeline in non blocking mode  

            message : string 
                String obtained in the socket.      
        """

        message = ""
        try:
            #Blocking mode
            if block_mode:
                # Get the message
                if sys.version_info[0] == 2:
                    info = self.sock.recv()
                else:
                    info = self.sock.recv_string()
                # Split the message

                index = info.find(' ')
                topic = info[0:index].strip()
                message = info[index+1:]
                success = True
            #Non blocking mode
            else:
                if self.transport ==  "ZMQ":
                    if sys.version_info[0] == 2:
                        info = self.sock.recv(flags = zmq.NOBLOCK)
                    else:
                        info = self.sock.recv_string(flags = zmq.NOBLOCK)
                elif self.transport ==  "NN" or self.transport ==  "nanomsg":
                    info = self.sock.recv(flags=nanomsg.DONTWAIT)
                    print(info)
                      
                # Split the message
                index = info.find(' ')
                topic = info[0:index].strip()
                message = info[index+1:]

                success = True
        #Exeption for non blocking mode timeout
        except zmq.Again as e:
            #Nothing to read
            success = False
            pass

        #Exeption for non blocking mode timeout
        except nanomsg.NanoMsgAPIError:
            #Nothing to read
            success = False
            pass

        return  success, message
        
    def listen_json(self, block_mode =  False):
        """ Listen for a json message. The operation is by default in non blocking mode
            
            Parameters
            ----------

            block_mode : bool
                If True, the socket will set in blocking mode, otherwise the socket will be in non blocking mode
                
            Returns
            -------

            success : bool
                If True the information was obtained inside the timeline in non blocking mode  

            info : dictionary
                Message obtained
        """
         
        success = False
        info = {}
        try:
            #Blocking mode
            if block_mode:
                success, info = self.__deserialization(self.sock.recv())
                #time.sleep(.001)
            #Non blocking mode
            else:
                if self.transport ==  "ZMQ":
                    if sys.version_info[0] == 2:
                        success, info = self.__deserialization(self.sock.recv(flags = zmq.NOBLOCK))
                    else:
                        success,info = self.__deserialization(self.sock.recv_string(flags = zmq.NOBLOCK))

                elif self.transport ==  "NN" or self.transport ==  "nanomsg":
                    success, info = self.__deserialization(self.sock.recv(flags=nanomsg.DONTWAIT))

                #time.sleep(.001)
        #Exeption for non blocking mode timeout
        except zmq.Again as e:
            #Nothing to read
            success = False
            pass

        #Exeption for non blocking mode timeout
        except nanomsg.NanoMsgAPIError:
            #Nothing to read
            success = False
            pass

        return  success, info


if __name__ == "__main__":
    import doctest
    doctest.testmod()



# coding = utf-8
#!/usr/bin/env python

# ------------------------------ Publisher ---------------------------------
# Description: Low-level Publisher Class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import zmq
import simplejson
import sys
import nep



class publisher:
    """ Publisher class used for inter-process comunication between nodes. Supports ZeroMQ, nanomsg and ROS publishers. 
        
        Parameters
        ----------

        topic : string 
            Topic name to publish the messages

        node : string or ROS node
            Name of the node or ROS 2.0 instance

        msg_type : string
            Message type: "string", "json" for ZMQ and nanomsg, some ROS geometry messages are also supported 

        conf : dictionary 
            Configuration of the publisher.

        debug: bool
            If True some additional information of the publisher is shown

    """

    def __init__(self, topic, node = "default", msg_type = "json", conf =  {'transport': "ZMQ", 'network': "broker", 'mode':"many2many"}, debug = False):

        self.conf = conf
        self.network = self.conf['network']
        self.transport =  self.conf['transport']
        self.node = node
        self.topic = topic
        self.msg_type =  msg_type
        self.ip = ""
        self.port = ""
        self.connected = False
        self.debug = debug

        if self.transport ==  "ROS":                                                    #Use ROS
            print ("PUB: " + self.topic + " using ROS 1.0")
            self.__create_ROS_publisher()

        elif self.transport ==  "ROS2":                                                   #Use ROS 2.0
            print ("PUB: " + self.topic + " using  ROS 2.0")
            self.__create_ROS2_publisher()

        elif self.transport ==  "ZMQ":                                                  #Use ZeroMQ
            print ("PUB: " + self.topic + " using ZMQ " + self.network)
            #many2many is the default value
            self.mode = "many2many"                                                     
            if "mode" in self.conf:
                self.mode = self.conf['mode']
            # Create a ZMQ socket
            self.__create_ZMQ_publisher()

        elif self.transport ==  "NN" or self.transport == "nanomsg":                    #Use nanomsg
            
            print ("PUB: " + self.topic + " nanomsg " + self.network)
            #many2many is the default value
            self.mode = "many2many" 
            if "mode" in self.conf:
                self.mode = self.conf['mode']
            self.__create_NN_publisher()
        else:
            msg = "NEP ERROR: Transport parameter " + self.transport + "is not supported, use instead 'ROS', 'ROS2', 'ZMQ' or 'nanomsg' "
            raise ValueError(msg)

    def __create_ROS2_publisher(self):

        from std_msgs.msg import String
        self.ros_pub = self.node.create_publisher(String, self.topic)
        self.connected = True



    #TODO:also define queue_size
    def __create_ROS_publisher(self):
        """Function used to create a ROS publisher"""
        import rospy
        
        from std_msgs.msg import String
        self.ros_pub = rospy.Publisher(self.topic, String, queue_size=10)

        rospy.init_node(self.node, anonymous = True)
        self.connected = True
        
    def __network_selection(self):
        """ Get IP and port of this socket

        Returns
        ----------

        success : bool
            Only if True socket can be connected

        port : string
            Port used to connect the socket

        ip : string
            IP used to connect the socket
        """
        success = False
        ip = "127.0.0.1"
        port = "8000"
        self.pid = os.getpid()
        if self.network == "direct":
            # Set the port and ip selected by the user
            port = self.conf["port"]
            ip = self.conf['ip']
            success =  True
        elif self.network == "broker":
            if self.topic != "/nep_node":
                print("PUB: " + self.topic + " waiting NEP master ...")
            # Register the topic in the NEP Master and get the port and ip
            success, port, ip  = nep.masterRegister(self.node, self.topic, master_ip = '127.0.0.1', master_port = 7000, socket = "publisher", mode = self.mode, pid = self.pid)
            if self.topic != "/nep_node":
                print("PUB: " + self.topic + " socket ready")
                
        return success, port, ip

    def __create_ZMQ_publisher(self):
        """Function used to create a ZeroMQ publisher"""

        success, self.port, self.ip = self.__network_selection()
        if success:    
            # Create a new ZeroMQ context and a publisher socket
            try:
                context = zmq.Context()
                # Define the socket using the "Context"
                self.sock = context.socket(zmq.PUB)
                #Set the topic of the publisher and the end_point
                self.__connect_ZMQ_socket()
                self.connected = True
            except:
                print ("NEP ERROR: socket already in use")
            
            time.sleep(1)
            #This delay in important, whithout them the comunication is not effective
 
            # ZeroMQ note:
            # There is one more important thing to know about PUB-SUB sockets: 
            # you do not know precisely when a subscriber starts to get messages.
            # Even if you start a subscriber, wait a while, and then start the publisher, 
            # the subscriber will always miss the first messages that the publisher sends. 


            # In Chapter 2 - Sockets and Patterns we'll explain how to synchronize a 
            # publisher and subscribers so that you don't start to publish data until 
            # the subscribers really are connected and ready. There is a simple and 
            # stupid way to delay the publisher, which is to sleep. Don't do this in a
            #  real application, though, because it is extremely fragile as well as
            #  inelegant and slow. Use sleeps to prove to yourself what's happening, 
            # and then wait for 
            # Chapter 2 - Sockets and Patterns to see how to do this right

    def __create_NN_publisher(self):
        """Function used to create a Nanomsg publisher"""
        try:
            import nanomsg
            self.NN_installed = True
        except ImportError:
            print ("NEP WARNING: Nanomsg not installed")
            self.NN_installed = False
        if self.NN_installed == False:
            msg = "Unable to use surveyor pattern due that Nanomsg is not installed "
            raise ValueError(msg)
        else:
            success, self.port, self.ip = self.__network_selection()
            if success:   
                try:
                    self.sock = nanomsg.Socket(nanomsg.PUB)
                    self.__connect_NN_socket()
                    self.connected = True
                    time.sleep(1)
                except Exception as e:
                        exc_type, exc_obj, exc_tb = sys.exc_info()
                        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                        print(exc_type, fname, exc_tb.tb_lineno)
          

    def __connect_ZMQ_socket(self):
        """ Connect ZMQ socket in base it configuration
        """
        endpoint = "tcp://" + self.ip + ":" + str(self.port)
        if self.mode == "one2many":
        # This allows only use one publisher connected at the same endpoint
            self.sock.bind(endpoint)
            if self.debug or self.network == "direct":
                if not self.topic == "/nep_node":
                    print("PUB: " + self.topic + " endpoint: " + endpoint +  " bind")
        elif self.mode == "many2one":
            # This allows two use more that one publisher ate the same endpoint
            self.sock.connect(endpoint)
            if self.debug or self.network == "direct":
                if not self.topic == "/nep_node":
                    print("PUB: " + self.topic + " endpoint: " + endpoint +  " connect")
        elif  self.mode == "many2many":
            self.sock.connect(endpoint)
            if self.debug or self.network == "direct":
                print("PUB: " + self.topic + " endpoint: " + endpoint +  " connect")

    def __connect_NN_socket(self):
        """ Connect Nanomsg socket in base it configuration
        """
        if self.mode == "one2many":
            # This allows only use one publisher connected at the same endpoint
            if self.ip == '127.0.0.1':
                ip = "*"
            endpoint = "tcp://" + ip + ":" + str(self.port)
            self.sock.bind(endpoint)
            #if self.debug or self.network == "direct":
                #print ("publisher " + endpoint +  " bind")
        
        elif self.mode == "many2one":
            # This allows two use more that one publisher ate the same endpoint
            if self.ip == '127.0.0.1':
                ip = "localhost"
            endpoint = "tcp://" + ip + ":" + str(self.port)
            self.sock.connect(endpoint)
            #if self.debug or self.network == "direct":
                #print ("publisher " + endpoint +  " connect")
            
        elif  self.mode == "many2many":
            if self.ip == '127.0.0.1':
                ip = "localhost"
            endpoint = "tcp://" + ip + ":" + str(self.port)
            self.sock.connect(endpoint)
            #if self.debug or self.network == "direct":
                #print ("publisher " + endpoint +  " connect")
              
        

    def __dumps(self,o, **kwargs):
        """Serialize object to JSON bytes (utf-8). See jsonapi.jsonmod.dumps for details on kwargs.

        Returns
        -------
        message : string
            Encoded string 

        """

        if 'separators' not in kwargs:
            kwargs['separators'] = (',', ':')
        
        s = simplejson.dumps(o, **kwargs)
        

        if sys.version_info[0] == 2: #Python 2
            if isinstance(s, unicode):
                s = s.encode('utf8')
        return s


    def __serialization(self, message):
        """ Function used to serialize a python dictionary using json. In this function the message to be send is attached to the topic. 
            
            Parameters
            ----------
            message : dictionary
                Python dictionary to be send
            
            Returns
            -------
            message : string
                Message to be send: topic + message 
        """
        return self.topic + ' ' + self.__dumps(message)
            
    def publish(self, message):
        """ Send message to subscribers

        Parameters
        ----------
        message : obj 
            Message to be send, must have the same type that msg_type 

        """
        
        if self.msg_type == "string":
            self.send_string(message)
        elif self.msg_type == "json":
            self.send_json(message)
        else:
            msg = "NEP ERROR: msg_type selected '" + str(self.msg_type) + "' non compatible"
            raise ValueError(msg)
    
    def send_string(self,message):
        """ Publish a string value. 
            
            Parameters
            ----------
            message : string 
                String to be sended 
        """
        if self.connected:

            if self.transport ==  "ZMQ":
                self.sock.send_string(self.topic + ' ' + message)
            
            elif self.transport ==  "NN" or self.transport ==  "nanomsg":
                self.sock.send(self.topic + ' ' + message)

            elif self.transport == "ROS":
                self.ros_pub.publish(message)

            elif self.transport == "ROS2":

                from std_msgs.msg import String
                msg = String()
                msg.data = message
                self.ros_pub.publish(msg)

        else:
            print ("NEP ERROR: socket already in use, message not sent")


    def send_json(self, message):
        """ Function used to publish a python dictionary. The message is serialized using json format and then published by the socket
            
            Parameters
            ----------
            message : dictionary 
                Python dictionary to be send

        """
        if self.connected:

            if self.transport ==  "ZMQ" or self.transport ==  "NN" or self.transport ==  "nanomsg":
                
                info = self.__serialization(message)

                if sys.version_info[0] == 2: #Python 2
                    self.sock.send(info)
                else: # Python 3
                    self.sock.send_string(info)

            elif self.transport == "ROS":
                message = self.__dumps(message)
                self.ros_pub.publish(message)

            elif self.transport == "ROS2":
                message = self.__dumps(message)
                from std_msgs.msg import String
                msg = String()
                msg.data = message
                self.ros_pub.publish(msg)

        else:
            print ("NEP ERROR: socket already in use, message not sent")
        

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# coding = utf-8
#!/usr/bin/env python

# ------------------------------- Node -------------------------------
# Description: Low-level Node Class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga

import time
import os
import sys
import signal
import threading
import nep
import zmq
import atexit
from zmq.eventloop import ioloop, zmqstream



#TODO: bug sending strings there is a space + message in publish-subscriber with ZMQ
#TODO: unregister topic when closed


try:
    if sys.version_info[0] == 2:
        import nanomsg
except ImportError:
    pass

try:
    if sys.version_info[0] == 3:
        import rclpy
except ImportError:
    pass   

class node:

    # ----------------------- __signal_handler  ------------------------------
    def __signal_handler(self, signal, frame):
        """ Exit node with Ctrl+C """
        import os
        print('Signal Handler, you pressed Ctrl+C! to close the node')
        if not os.environ.get('OS','') == 'Windows_NT': # Windows
            time.sleep(.5)
            os.system('kill %d' % os.getpid())
            sys.exit(0)
        else:
            """Signal handler used to close when user press Ctrl+C"""
            time.sleep(.5)
            import os
            pid = os.getpid()
            import subprocess as s
            s.Popen('taskkill /F /PID {0}'.format(pid), shell=True)
            sys.exit(0)
        
    # ----------------------- __wait_kill ------------------------------
    def __wait_kill(self):
        """Listen for close the current node from an extrenal message on the topic \nep_node"""
        exit = False
        time.sleep(.1)
        conf = self.direct("127.0.0.1", "12345", "one2many") 
        exit_sub  = self.new_sub("/nep_node", "json", conf)
        time.sleep(.1)

        # Always wait in blocking mode for the kill message of the node 
        while not exit:
            # Wait for kill signal (Operation in blocking mode)
            s, data = exit_sub.listen_json(True) 
            # If new kill signal detected
            if s == True:
      
                try:
                    kill_proccess = False

                    if "node" in data:
                        node_ = data["node"]
                        if type(node_) is list:
                            for l in node_:
                                # If one of the names and the type of the request to kill is equal the name and type of this node, then kill the node
                                if l == self.node_name:
                                    kill_proccess = True
                        elif type(node_) is str: 
                            # If current name and type of the request to kill is equal the name and type of this node, then kill the node
                            if node_ == "all":
                                kill_proccess = True
                            if node_ == self.node_name:
                                kill_proccess = True

                    if kill_proccess:
                        self.__unregister()
                        print ("************* Node signal **************")
                        print (data)
                            
                        exit = True
                        import os

                        if not os.environ.get('OS','') == 'Windows_NT': # Windows
                            os.system('kill %d' % os.getpid())
                            sys.exit(0)
                        else:
                            pid = os.getpid()
                            print (pid)
                            import subprocess as s
                            s.Popen('taskkill /F /PID {0}'.format(pid), shell=True)
                except:
                    print ("NEP ERROR: processing node signal")
                    pass

    def __unregister(self):
        # TODO send pub message to manster to unregister topics
        print ("Closing NEP node")
        time.sleep(3)
        pass
        
    
    def __init__(self, node_name, transport = "ZMQ", exit_thread = True):
        """
        Class used to define a new node using the publisher-subscriber pattern. This class is compatible with ZeroMQ, Nanomsg and ROS.

        Parameters
        ----------

        node_name : string 
            Name of the node
        
        transport : string
            Define the transport layer of the node, can be 'ROS' to use ROS, 'ZMQ' to use ZeroMQ and 'NN' or 'nanomsg' to use nanomsg.

        exit_thread : bool
            If True then the node can be killed sending a dictionary to the "/nep_node" topic with the info of {'node': <node-name-to-kill>} or {"type":<node-type-to-kill>}. Where  <node-name-to-kill> and  <node-type-to-kill> can be string or list of strings.

        """
        atexit.register(self.__unregister)
        self.NN_installed = False
        try:
            import nanomsg
            self.NN_installed = True
        except ImportError:
            self.NN_installed = False
            
        # Enable to kill the node using Ctrl + C
        signal.signal(signal.SIGINT, self.__signal_handler)
        
        self.node_name = node_name
        self.transport  = transport
        self.pid = os.getpid()

        print ("NODE: " + self.node_name + ", pid: " + str(self.pid))

        if self.transport == "ZMQ" or self.transport == "NN" or self.transport == "nanomsg":
            if exit_thread: # Enable this node to be killed from an external signal
                
                # ------------------------- Kill thread ------------------------------------
                # thread that can de used to stop the program
                self.exit = threading.Thread(target = self.__wait_kill)
                # Used to finish the background thread when the main thread finish
                self.exit.daemon = True
                # start new thread 
                self.exit.start()
        
        # Import ROS libraries and create new ROS enviroment

        if self.transport  == 'ROS':
            try:
                import rospy		
                rospy.init_node(node_name, anonymous=True)
                print ("ROS node started")
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)

                print ("NEP ERROR: rospy not found, possible causes:")
                print ("- Are you using Ubuntu or another Linux distribution complatible with ROS?")
                print ("- Have you installed ROS in your PC?")
                print ("- Is ROS installed correctly ?")
                print ("- Are you using Python 2.7")
                
                if sys.version_info < (3.0):
                    y = raw_input("Press ENTER to continue and exit")
                else:
                    time.sleep(5)
                sys.exit(1)

        elif self.transport  == 'ROS2':
            try:
                import rclpy		
                rclpy.init()
                self.node_ros2 = rclpy.create_node(node_name)
                print ("NODE: ROS2 started")

            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)

                print ("NEP ERROR: rclpy not found, possible causes:")
                print ("- Have you installed ROS 2.0 in your PC?")
                print ("- Is ROS 2.0 installed correctly?")
                print ("- Are you using Python 3")
                
                if sys.version_info < (3.0):
                    y = raw_input("Press ENTER to continue and exit")
                else:
                    time.sleep(5)
                sys.exit(1)



    def broker(self, mode = "many2many"):
        """ 
        Publisher-Subscriber broker configuration
       
        Parameters
        ----------

        mode : string
            Only for ZeroMQ and Nanomsg. It can be "one2many" (one publisher and many subscribers in a topic), "many2one" (one publisher and many subscribers in a topic), "many2many" (many publishers and many subscribers in a topic).    

        Returns
        ----------

        conf: dictionary
            Dictionary with the specifications of the publisher

        """
        conf = {'transport': self.transport, 'network': "broker", 'mode':mode}
        return conf


    def direct(self, ip = "127.0.0.1", port = "9000", mode = "one2many"):

        """
        Publisher-Subscriber direct network configuration

        Parameters
        ----------

        port : string 
            Value of the port to perform the socket connection (only for ZeroMQ and Nanomsg in a "direct" network)

        ip : string 
            Value of the ip to perform the socket connection (only for ZeroMQ and Nanomsg in a "direct" network)

        mode : string
            Only for ZeroMQ and Nanomsg. It can be "one2many" (one publisher and many subscribers in a topic) or "many2one" (one publisher and many subscribers in a topic).

        Returns
        ----------

        conf: dictionary
            Dictionary with the specifications of the publisher

        """
        
        conf = {'transport': self.transport, 'network': "direct", 'port': port, 'ip': ip, 'mode':mode}
        return conf

    def new_client(self, topic):
        """
        Function used to generate a new client by specifying a topic

        Parameters
        ----------
        topic : string 
            Name of the topic

        Returns
        ----------

        client : nep.client
            Client instance

        """

        print("CLIENT: " + topic + ", waiting NEP master ...")
        s, port, ip  = nep.masterRegister(self.node_name, topic, master_ip = '127.0.0.1', master_port = 7000, socket = "client", pid = self.pid)

        if s:
            print ("CLIENT: " + topic + ", in " + ip + ":" + str(port))
            client = nep.client(ip, port, debug = False)
            print("CLIENT: " + topic + ", socket ready")
            return client
        else:
            print ("NEP ERROR: " + topic + ", client socket not connected")

    def new_server(self, topic):
        """
        Function used to generate a new server by specifying a topic

        Parameters
        ----------
        topic : string 
            Name of the topic

        Returns
        ----------

        server : nep.server
            Server instance

        """

        print("SERVER: " + topic + ", waiting NEP master ...")
        s, port, ip  = nep.masterRegister(self.node_name, topic, master_ip = '127.0.0.1', master_port = 7000, socket = "server", pid = self.pid)
        if s:
            print ("SERVER: " + topic + ", in " + ip + ":" + str(port))
            server = nep.server(ip,port, debug = False) #Create a new server instance
            print("SERVER: " + topic + ", socket ready")
            return server
        else:
            print ("NEP ERROR: " + topic + ", server socket not connected")
 

    def new_pub(self,topic, msg_type = "json", configuration =  {'transport': "ZMQ", 'network': "broker", 'mode':"many2many"}):
        """
        Function used to generate a new publisher in the current node

        Parameters
        ----------
        topic : string 
            Name of the topic

        configuration : dictionary 
            Configuration of the publisher


        Returns
        ----------

        pub : nep.publisher
            Publisher instance

        """
        if self.transport == 'ROS2':
            configuration =  {'transport': "ROS2", 'network': "broker", 'mode':"many2many"}
            pub = nep.publisher(topic, self.node_ros2, msg_type,configuration )
            return pub

        if self.transport == 'ROS':
            configuration =  {'transport': "ROS", 'network': "broker", 'mode':"many2many"}
            
        #TODO: launch error if you dont put msg_type or conf
        pub = nep.publisher(topic, self.node_name, msg_type, configuration)
        return pub

    def new_callback(self, topic , msg_type , callback, conf = {'transport': 'ZMQ', 'network': "broker", 'mode':"many2many"}):
        """
        Function used to generate a new subscriber callback. Only for ZMQ and ROS.

        Parameters
        ----------
        topic : string 
            Name of the topic

        msg_type : string
            Message type: "string", "json" for ZMQ and nanomsg, some ROS geometry messages are also supported 

        callback: function
            Function that will be executed

        """

        if self.transport == 'ROS2':
            conf = {'transport': "ROS2", 'network': "broker", 'mode':"many2many"}
            self.sub = nep.subscriber(topic, self.node_ros2, msg_type, conf, callback)

        elif self.transport  == 'ROS':
            conf = {'transport': "ROS", 'network': "broker", 'mode':"many2many"}
            self.sub = nep.subscriber(topic, self.node_name, msg_type, conf, callback)

        elif self.transport  == 'ZMQ':
            self.sub = nep.subscriber(topic, self.node_name, msg_type, conf)
            self.sub_socket = self.sub.sock
            self.sub_callback = callback
        else:
            print("NEP ERROR: middleware " + self.transport + " do not support callback subscribers")

        return self.sub

    def __process_message(self,msg):
        """Get message only"""
        info = msg[0]
        index = info.find(' ')
        topic = info[0:index].strip()
        message = info[index+1:]
        self.sub_callback(message)

    def spin(self):
        """ Start event loop"""
        if self.transport  == 'ZMQ':
            # Event Loop calback option
            if self.sub_callback != None:
                stream_sub = zmqstream.ZMQStream(self.sub_socket)
                stream_sub.on_recv(self.__process_message)
                ioloop.IOLoop.instance().start()
        elif self.transport  == 'ROS':
            import rospy
            """Function used to spin ROS subscriber"""
            rospy.spin()

        elif  self.transport  == 'ROS2':
            import rclpy
            while rclpy.ok():
                rclpy.spin_once(self.node_ros2)
            # Destroy the node explicitly
            # (optional - otherwise it will be done automatically
            # when the garbage collector destroys the node object)
            self.node_ros2.destroy_node()
            rclpy.shutdown()


    def new_surveyor(self, topic, timeout = 1000):
        """
        Function used to generate a new subscriber in the current node

        Parameters
        ----------
        topic : string 
            Name of the topic

        timeout : int
            Maximun miliseconds waiting for response

        Returns
        ----------

        sub : nep.surveyor 
            subscriber instance
        """
        sur = nep.surveyor(topic, timeout, self.node_name)
        return sur

    def new_respondent(self, topic, msg_type = "json"):
        """
        Function used to generate a new subscriber in the current node

        Parameters
        ----------
        topic : string 
            Name of the topic

        msg_type : Type of message
            Name of the topic


        Returns
        ----------

        sub : nep.respondent 
            subscriber instance
        """
        res = nep.respondent(topic, self.node_name)
        return res


    def new_sub(self, topic, msg_type = "json", configuration =  {'transport': "ZMQ", 'network': "broker", 'mode':"many2many" }):

        """
        Function used to generate a new subscriber in the current node

        Parameters
        ----------
        topic : string 
            Name of the topic

        msg_type : Type of message
            Name of the topic

        configuration : dictionary  
            Configuration of the subscriber

        Returns
        ----------

        sub : nep.subscriber 
            subscriber instance
        """

        sub = nep.subscriber(topic, self.node_name, msg_type, configuration)
        return sub


# TODO: sock.close() and context.destroy() must be set when a process ends
# In some cases socket handles won't be freed until you destroy the context.
# When you exit the program, close your sockets and then call zmq_ctx_destroy(). This destroys the context.
# In a language with automatic object destruction, sockets and contexts 
# will be destroyed as you leave the scope. If you use exceptions you'll have to
#  do the clean-up in something like a "final" block, the same as for any resource.


if __name__ == "__main__":
    import doctest
    doctest.testmod()



import nep
import threading
import time
import signal
import sys, os

class robot:
    
    def __init__(self, robot_name, pattern = "survey"):
        """ Define a new robot

            Parameters
            ----------

            robot_name : string
                Node or robot name which identify the robot

            pattern : string
                - If selected "survey" or "surveyor", then "nanomsg" will be used as middleware
                - If selected "actionlib", then "ROS" will be used as middleware
                - If selected "client-server", then "ZeroMQ" will be used as middleware
                - If selected "pub-sub", then  "ZeroMQ" will be used as middleware
        """

        self.middleware = pattern
        self.robot_name = robot_name
        self.deadline = 60 # cancel action that last more that 60 seconds
        self.action = {"state":"success"}
        self.currentActionID = "0"
        self.node = nep.node(robot_name, "ZMQ", True)

        conf_pub = self.node.broker()
        self.sharo = self.node.new_pub("/sharo", "json", conf_pub)

        # Use survey pattern
        if self.middleware == "nanomsg":
            print ("ROBOT NODE using: survey pattern")
            self.resp = self.node.new_respondent("action_survey") # Respondent subscribes to /manager topic 
        
        elif self.middleware == "ZMQ":
            print ("ROBOT NODE using: client-server pattern")
            self.server = self.node.new_server("action_server") # Respondent subscribes to /manager topic 

        elif self.middleware == "ROS":
            print ("ROBOT NODE using: ROS 1.0 actionlib")
            pass

        elif self.middleware == "ROS2":
            print ("ROBOT NODE using: ROS 2.0 actionlib")
            pass

        time.sleep(1)
    
        self.state = "idle" # idle can only be idle, busy or wait
        self.result = "inactive" # result can only be success or failure.
        self.action2do = {"status":"0", "action":{}}

        # Give feedback to the rize interface
        self.robot_actions = {}
        self.robot_cancel_actions = {}

        self.isCanceled = False
        self.inActionExecution = False
        self.running = True

        # ------------------------- Action thread ------------------------------------
        self.action_thread = threading.Thread(target = self.onActionLoop)
        self.action_thread.start()
        
        # ------------------------- Stop thread ------------------------------------
        self.cancel_thread = threading.Thread(target = self.onCancelLoop)
        self.cancel_thread.start()

    def onActionLoop(self):

        while self.running:    
            if self.action["state"] == "active" or self.action["state"] == "pending":
                self.inActionExecution = True #TODO: lock
                self.run_action(self.action)
                self.sharo.publish(self.action)
                print (self.action["id"] + " finished")
                self.inActionExecution = False #TODO: lock
                self.action["state"] = "success"
            time.sleep(.001)

    def onCancelLoop(self):
        while self.running:
           
            if self.isCanceled:
                print ("CANCEL" + str(self.action))
                self.run_action(self.action, cancel = True)
                self.isCanceled = False 
                print ("CANCELLLLLLL ---------------------------")
            else:
                time.sleep(0.05)


    def setRobotActions(self,robot_actions):
        """ Set a python dictionary with all the functions which the node can execute
            Parameters
            ----------

            robot_actions : dictionary
                Dictionary of functions
            
        """
        try:
            self.robot_actions = robot_actions
        except Exception as e: 
            print(e)
            print ("Error setting actions")
            self.node.failure()
            time.sleep(1)

    
    def wait(self, input_ = 0,  parameters = {}, parallel = False ):
        """ Generic wait primitive
            Parameters
            ----------

            input : float
                Seconds to wait
            
        """
        try:
            value = float(input_)
            time.sleep(value)
        except:
            pass
        return True


    def setCancelActions(self,actions):
        """ Set a python dictionary with all the functions which the node can execute
            Parameters
            ----------

            robot_actions : dictionary
                Dictionary of functions
            
        """
        try:
            self.robot_cancel_actions = actions
        except Exception as e: 
            print(e)
            print ("Error setting cancel actions")
            self.node.failure()
            time.sleep(1)
        

                      
    def __listen_request(self):
        """ Listen and wait for the robot actions
        """
        success =  False
        action_request = {}
        if self.middleware == "nanomsg":
            success, action_request = self.resp.listen_json(block_mode=False)
            #if success:
                #print (action_request)
        elif self.middleware == "ZMQ":
            success =  True
            action_request = self.server.listen_info()

        if success:
            #print (action_request)
            if action_request["node"] == "cancel":
                return  True, action_request
            
            if self.robot_name in action_request['robots']: 
                return  True, action_request
            else:
                return  False, action_request

        return success, action_request


    def __send_response(self,msg):
        """ Send the response to the survey/cognitive node
        """
        #print 
        #print ("Response:")
        #print (msg)
        if self.middleware == "nanomsg":
            self.resp.send_json(msg)
        elif self.middleware == "ZMQ":
            self.server.send_info(msg)

    
        
       
    def run(self):
        while self.running:

            success, action_request =  self.__listen_request()

            if success:
                
                if action_request["node"] == "cancel":
                    self.isCanceled = True
                    msg = {"node":"canceled", "robot":self.robot_name}
                    self.__send_response(msg)
                else: 
                    
                    listPrimitives = []
                    primitives = action_request["primitives"]
                    for p in primitives:
                        listPrimitives.append(p["primitive"])

                    #print ("ACTION: --" +  action_request["id"] + "-- primitives: " + str(listPrimitives) )  

                    # If an action is in execution
                    if self.inActionExecution: 
                        
                        # Check if the action request is new or the same
                        new_action = self.checkIfNew(action_request)
                        #print ("returned new action" + str(new_action))
                        
                        # If is a new action, the cancel current action and return pending
                        if new_action:
                            #print ("new action - cancel")
                            self.isCanceled = True #TODO lock
                            self.execution_state = "pending"
                        # Else return running, action still not finished
                        else:
                            self.execution_state = "running"

                    # If robot idle
                    else:
                        new_action = self.checkIfNew(action_request)
                        #print ("returned new action " + str(new_action))
                        if new_action:
                            #print ("new action  - run")
                            #print ("running " + action_request["id"])
                            self.resetActionTime() #Reset time counter for actions
                            self.setAction(action_request) #Set new action to execute
                            self.execution_state = "running"
                        else:
                            self.execution_state = "success"

                    msg = {"node":self.execution_state, "robot":self.robot_name}
                    if self.execution_state == "success":
                        print ("Action -- " + action_request["id"] + " -- success")
                    #print (msg)

                    self.__send_response(msg)
                
                
            if self.inActionExecution:
                action_time = self.getActionTime()
                if action_time > self.deadline: # Cancel if is in a posible deadlock
                    self.isCanceled = True #TODO lock
            time.sleep(0.01)

                

    def resetActionTime(self):
        self.initActionTime =  time.time()

    def setAction(self,action_request):
        self.currentActionID = action_request["id"]
        self.action = action_request

    def getActionTime(self):
        return time.time() - self.initActionTime 


    def checkIfNew(self,action_request):
        id_ = action_request["id"]

        #print (self.currentActionID)
        if id_ != self.currentActionID:
            return True
        else:
            return False
            
        

    def run_action(self, message, cancel = False):
        """ Run an action
            
            Parameters
            ----------

            action : dictionary
                Action description

        """
            
        action = message["primitives"] # Get list of concatenated primitives
        n_primitives = len(action)
        in_parallel = True
                
        # Perform all the actions in parallel
        for i in range(n_primitives):
                    
            # Except the last one
            if (i == n_primitives-1):
                in_parallel = False
                
            primitive = action[i]
            primitive_name = primitive["primitive"] #Name of the primitive
            input_ = primitive["input"]
            options = primitive["options"]

            if primitive_name in self.robot_actions:
                # Execute function
                if cancel:
                    try:
                        self.robot_cancel_actions[primitive_name]()
                        print (primitive_name, " I am cancelling")
                    except:
                        pass
                else:
                    try:
                        self.robot_actions[primitive_name](input_, options, in_parallel)
                        print (primitive_name, " was executed")
                    except:
                        print ("NEP ERROR: "  + " primitive " + primitive_name + " was not executed")
            else:
                print (primitive_name, "is not a valid primitive")



# coding = utf-8
#!/usr/bin/env python

# ------------------------------ Helper functions ---------------------------------
# Description: Some useful functions used in NEP core
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


from os import listdir
from os.path import isfile, join
import simplejson
import nep
import time
import sys, os
from subprocess import Popen, call


def nepmaster(version="2", param =""):
    """
        Launch master in NEP_WS

        Parameters
        ----------
        version : string
            Python version, 0 for default, 2 for Python 2 and 3 for Python 3

        param : string
            Can be "local", or "network"
    """

    if os.environ.get('OS','') == 'Windows_NT':
        from subprocess import CREATE_NEW_CONSOLE

    nep_ws = nep.getNEPpath()
    script = "master"
    command = "python " + nep_ws + "/" + script + ".py"+ " " + param

    if os.environ.get('OS','') == 'Windows_NT':

        if version == "2": #If more that one python version, specify Python 2
            print ("Running in Python 2")
            command = "py -2 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters
        elif version == "3": #If more that one python version, specify Python 3
            print ("Running in Python 3")
            command = "py -3 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters

        print ("Windows launcher in new console .......")
        Popen(command, creationflags=CREATE_NEW_CONSOLE)
    else: 
        print ("OSX launcher .......")

        if version == "2": #If more that one python version, specify Python 2
            print ("Running in Python 2")
            command = "python2 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters
        elif version == "3": #If more that one python version, specify Python 3
            print ("Running in Python 3")
            command = "python3 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters

        import applescript

        tell = 'tell application "Terminal" to do script '
        complete = tell + '"' +  command + '"'

        applescript.AppleScript(complete).run()


def neprun(module, script, parameters, version="2"):

    """
        Launch a python script in NEP_WS

        Parameters
        ----------
        module : string
            Module name

        script : string
            Script name

        script : parameters
            Additional command line parameters

        version : string
            Python version, 0 for default, 2 for Python 2 and 3 for Python 3

    """
    try:

        if os.environ.get('OS','') == 'Windows_NT':
            from subprocess import CREATE_NEW_CONSOLE

        nep_ws = nep.getNEPpath() 
        command = "python " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters
    


        print ("To run: " + command)
        if os.environ.get('OS','') == 'Windows_NT':

            if version == "2": #If more that one python version, specify Python 2
                print ("Running in Python 2")
                command = "py -2 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters
            elif version == "3": #If more that one python version, specify Python 3
                print ("Running in Python 3")
                command = "py -3 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters

            print ("Windows launcher in new console .......")
            Popen(command, creationflags=CREATE_NEW_CONSOLE)
        else: 
            print ("OSX launcher .......")

            if version == "2": #If more that one python version, specify Python 2
                print ("Running in Python 2")
                command = "python2 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters
            elif version == "3": #If more that one python version, specify Python 3
                print ("Running in Python 3")
                command = "python3 " + nep_ws + "/" + module + "/" + script + ".py"+ " " + parameters

            import applescript

            tell = 'tell application "Terminal" to do script '
            complete = tell + '"' +  command + '"'

            applescript.AppleScript(complete).run()

    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno)
        time.sleep(3)
        return False
 

def masterRegister(node, topic, master_ip = '127.0.0.1', master_port = 7000, socket = "subscriber", mode = "many2many", pid = "none"):
    """ Register topic in master node
            
    Parameters
    ----------

    node: string
        Node name

    topic : string
        Topic to register

    master_ip : string 
        IP of the master node service

    master_port : int
        Port of the master node service

    socket: string
        Socket type. Example "surveyor", "publisher", "subscriber", "respondent", "client", "server"

    mode: string
        Parameter only for Publish/Subscriber pattern. Options are "one2many", "many2one" and "many2many".

    Returns
    ----------

    result : bool
        Only if True socket can be connected

    port : string
        Port used to connect the socket

    ip : string
        IP used to connect the socket
    
    """
    topic = topic
    client = nep.client( master_ip, master_port, transport = "ZMQ", debug = False)
    time.sleep(.01)
    message = {
            'node':node,
            'topic':topic,
            'mode':mode,
            'socket':socket,
            'pid':pid
              }
    
    client.send_info(message)
    response = client.listen_info()
    try:
        topic_id = response['topic']

        if(topic_id == topic):
            port = response['port']
            if "ip" in response:
                ip = response['ip']
            else:
                ip = '127.0.0.1'
            state = response['state']
            if state == "success":
                return True, port, ip
            else:
                print ("NEP ERROR: wrong socket configuration")
                return False, port, ip
    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno)
        print ("NEP ERROR: wrong response from master")
        return False, "none", "none"

    print ("NEP ERROR: wrong topic from master")
    return False, "none", "none"

def getNEPpath():
    """ Get path to NEP Workspace

    Returns
    ----------

    path : string
        Current workspace path

    """
    import os
    return os.environ['NEP_WS']

def getPath():
    return os.getcwd()


def setNEPpath(new_path):
    """ Set path to NEP Workspace

    Parameters
    ----------

    new_path: string
        New path for NEP workspace

    """
    import os

    if os.environ.get('OS','') == 'Windows_NT':
        from subprocess import CREATE_NEW_CONSOLE
        command = 'setx NEP_WS "' + new_path + '"'
        Popen(command, creationflags=CREATE_NEW_CONSOLE)
        
    os.environ["NEP_WS"] = new_path


def getMyIP():
    """ Get current IP address of the PC

    Returns
    ----------

    ip : string
        Current IP

    """
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    cw_ip = s.getsockname()[0]
    s.close()
    return str(cw_ip)

def json2dict(s, **kwargs):
    """Convert JSON to python dictionary. See jsonapi.jsonmod.loads for details on kwargs.
     
        Parameters
        ----------
        s: string
            string with the content of the json data

        Returns:
        ----------
        dict: dictionary
            dictionary with the content of the json data
    """

    if sys.version_info[0] == 3:
        return simplejson.loads(s, **kwargs)

    else:
        if str is unicode and isinstance(s, bytes):
            s = s.decode('utf8')
    
    return simplejson.loads(s, **kwargs)

def dict2json(o, **kwargs ):
    """ Load object from JSON bytes (utf-8). See jsonapi.jsonmod.dumps for details on kwargs.
     
        Parameters
        ----------
        o: dictionary
            dictionary to convert
            

        Returns:
        ----------
        s: string
            string in json format

    """
        
    if 'separators' not in kwargs:
        kwargs['separators'] = (',', ':')
        
    s = simplejson.dumps(o, **kwargs)

    import sys
    if sys.version_info[0] == 3:
        if isinstance(s, str):
            s = s.encode('utf8')

    else:
        if isinstance(s, unicode):
            s = s.encode('utf8')
        
    return s

def read_json(json_file):
    """ Read a json file and return a string 
        
        Parameters
        ----------
        json file:string
            Path +  name + extension of the json file

        Returns:
        ----------
        json_data: string
            string with the content of the json data

    """
    json_data = open (json_file).read()
    return json_data

def getFiles(path):
    """ Get a list of files that are inside a folder
        
        Parameters
        ----------
        path: string
            path of the folder

        Returns:
        ----------
        onlyfiles: list 
            list of strings with the name of the files in the folder

    """
    onlyfiles = [f for f in listdir(path) if isfile(join(path, f))]
    return onlyfiles



from .helpers import*
from .client import*
from .server import*
from .publisher import*
from .subscriber import*
from .master import* 
from .node import*
from .broker import*
from .robot import*

import platform
if platform.system() == 'Linux' :
	pass
else:
	from .surveyor import*
	from .respondent import*




# coding = utf-8
#!/usr/bin/env python

# ------------------------------- Respondent -------------------------------
# Description: Low-level Responsent class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import simplejson
import sys
import nep

try:
    import nanomsg
except ImportError:
    pass

class respondent():
        
        def __init__(self, topic, node_name = "default", debug = False):
            
            """ Nanomsg surveyor class
            
            Parameters
            ----------

            topic : string
                Topic to exchange info

            debug: bool
                If True some additional information of the subscriber is shown

            """
            self.topic = topic
            print("RESP: " + self.topic + " waiting for NEP master ...")

            self.pid = os.getpid()
            success, port, ip  = nep.masterRegister(node_name, self.topic, master_ip = '127.0.0.1', master_port = 7000, socket = "respondent", pid = self.pid)
            print("RESP: " + self.topic + " socket ready")
            self.debug = debug
           
            if success:

                self.NN_installed = False
                try:
                    import nanomsg
                    self.NN_installed = True
                except ImportError:
                    print ("NEP ERROR: Nanomsg not installed")
                    self.NN_installed = False

                if self.NN_installed == False:
                    msg = "Unable to use surveyor pattern due that Nanomsg is not installed "
                    raise ValueError(msg)

                
                self.sock = nanomsg.Socket(nanomsg.RESPONDENT)
                endpoint = "tcp://" + ip + ":" + str(port)
                self.sock.connect(endpoint)
                time.sleep(1)
                if self.debug:
                    print ("respondednt started in: " + str(endpoint))
        
              
        def __dumps(self,o, **kwargs):
            """Serialize object to JSON bytes (utf-8).
                
            See jsonapi.jsonmod.dumps for details on kwargs.
            """

            if 'separators' not in kwargs:
                kwargs['separators'] = (',', ':')
        
            s = simplejson.dumps(o, **kwargs)
            

            if sys.version_info[0] == 2: #Python 2
                if isinstance(s, unicode):
                    s = s.encode('utf8')
            return s


        #Status: OK
        def __serialization(self, message):
            """ Function used to serialize a python dictionary using json. 

                Parameters
                ----------
                message : dictionary
                    Python dictionary to be send

                    
                Returns
                -------
                message : string
                    Message to be send, topic + message 
            """
            return self.__dumps(message)
            
    

        def send_json(self, message):
            """ Function used to send a python dictionary.
                    
                Parameters
                ----------
                message : dictionary 
                    Python dictionary to be send

            """

            info = self.__serialization(message)
            self.sock.send(info)


        def __loads(self, s, **kwargs):
            """Load object from JSON bytes (utf-8). See jsonapi.jsonmod.loads for details on kwargs.
            """

            if sys.version_info[0] == 2:
                if str is unicode and isinstance(s, bytes):
                    s = s.decode('utf8')
        
            return simplejson.loads(s, **kwargs)


    
        #Status: OK
        def __deserialization(self, info):
            """ JSON deserialization function
                    
                Parameters
                ----------
                info : string
                    message received

                Returns
                -------
                info : dictionary
                    Message as python dictionary
            """
            try:
                json0 = info.find('{')
                topic = info[0:json0].strip()
                msg = self.__loads(info[json0:])
                success = True
            except:
                msg = ""
                success = False
            return msg


        #Status: OK
        def listen_json(self, block_mode = False):
            """ Listen for a json message
                        
                Returns
                -------

                info : dictionary
                    Message obtained
            """
          
            if block_mode:#TODO, change s, data= ....
                info = self.__deserialization(self.sock.recv())
                return  True, info
            else:
                try:
                     info = self.__deserialization(self.sock.recv(flags=nanomsg.DONTWAIT))
                     return True, info
                except nanomsg.NanoMsgAPIError:
                    return  False, {}


         
        
        def close(self):
            """Close socket """
            self.sock.close()
                

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# coding = utf-8
#!/usr/bin/env python

# -------------------------------- Client ---------------------------------
# Description: Low-level Client Class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import zmq
import simplejson
import sys
import nep
import socket


class client:
    def __init__(self, IP, port, transport = "ZMQ", debug = True):
        """
        Creates a new client object

        Parameters
        ----------

        IP : string
            IP value of server

        port : string
            Port used to connect the socket
            
        transport : string
            Define the transport layer of the server, can be 'ZMQ' or "normal", to use ZeroMQ or TCP/IP python sockets

        debug: bool
            If True some additional information of the socket is shown

        """
        
        
        self.transport = transport
        if self.transport == "ZMQ":
            context = zmq.Context()
            # Define the socket using the "Context"
            self.sock = context.socket(zmq.REQ)
            self.sock.connect("tcp://" + IP + ":" + str(port))
            if debug:
                print ("CLIENT: connected in " + IP + ":" + str(port))

        elif transport == "normal":

            # Normal sockets
            print ("CLIENT: POSIX server in " + IP + ":" + str(port))
            try:
                #create an AF_INET, STREAM socket (TCP)
                self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            except socket.error:
                print ('CLIENT: Failed creating socket')
                sys.exit()

            try:
                if IP == "localhost":
                #Same computer
                        IP = socket.gethostbyname( IP )
                        print ("CLIENT: local host in :", IP)
                
            except socket.gaierror:
                #could not resolve
                print ('Hostname could not be resolved. Exiting')
                sys.exit()
            #Connect to remote server
            max_v = 10
            i = 0
            connect=False
            while not connect:
                try: 
                    self.s.connect((IP , int(port)))
                    connect = True
                except:
                    print ("NEP WARNING: Server not found intent:" + str(i) + ", max = 10")
                    time.sleep(2)
                    i = i + 1
                    if i > max_v-1:
                        print ("NEP WARNING: Server not found after max number of intents")
                        time.sleep(4)
                        sys.exit()
            
            print ("CLIENT: connected in " + IP + ":" + str(port))



    def __loads(self, s, **kwargs):
        """Load object from JSON bytes (utf-8). See jsonapi.jsonmod.loads for details on kwargs.
        """
        
        if sys.version_info[0] == 2:
            if str is unicode and isinstance(s, bytes):
                s = s.decode('utf8')
        
        return simplejson.loads(s, **kwargs)

    
    #Status: OK
    def __deserialization(self, info):
        """ 
            Parameters
            ----------
            info : string
                topic + message received by the ZQM socket 

            Returns
            -------
            msg : string
                String message as a python dictionary
        """
        try:
            msg = self.__loads(info)
        except:
            msg = ""
        return msg

    def __dumps(self,o, **kwargs):
        """ JSON deserialization function  (string to dictionary)
            
            Parameters
            ----------
            info : string
                Message received by the ZQM socket 

            Returns
            -------
            info : dictionary
                Message as python dictionary
        """
        
        if 'separators' not in kwargs:
            kwargs['separators'] = (',', ':')
        
        s = simplejson.dumps(o, **kwargs)
        

        if sys.version_info[0] == 2: #Python 2
            if isinstance(s, unicode):
                s = s.encode('utf8')
        return s


    #Status: OK
    def __serialization(self, message):
        """ Function used to serialize a python dictionary using json. 

            Parameters
            ----------
            message : dictionary
                Python dictionary to be send
            
            Returns
            -------
            message : string
                Message to be send
        """
        return self.__dumps(message)



    def listen_info(self):
        """ Listen for a json message
                        
            Returns
            -------

            response : dictionary or string
                Message obtained
        """
        
        if self.transport == "ZMQ":
            #response = self.sock.recv_json()
            #return response
            response = self.sock.recv()
            return self.__deserialization(response)
            
        else:
          response = self.s.recv(1024)
          return response
            

    def send_info(self,request):
        
        """ Function used to send client request as a python dictionary  (if transport == "ZMQ") or as string (if transport == "normal")
            
            Parameters
            ----------

            request : dictionary or string
                    Python dictionary (ZMQ) or string (Python sockets) to be send

        """

        if self.transport == "ZMQ":
            #self.sock.send_json(request)
            if sys.version_info[0] == 2:
                self.sock.send(self.__serialization(request))
            else:
                self.sock.send_string(self.__serialization(request))

        else:
            try:
                self.s.sendall(request)
            except:
                print ("ERROR: for normal socket messages must be string not dictionaries")
                sys.exit()

if __name__ == "__main__":
    import doctest
    doctest.testmod()


# coding = utf-8
#!/usr/bin/env python

# ------------------------------- Master -----------------------------------
# Description: Low-level Master class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import zmq
import time
import nep
import sys, os
import signal


if sys.version_info[0] == 3:
    import _thread as thread
else:
    import thread
    
# ---------------------------------- Static ports ------------------------------------ :
# master port 7000

# ----------------------------------- Topic format ----------------------------------- :
# topic_name: [port,mode,ip] 

class master:

    def __signal_handler(self, signal, frame):
        import os


        print('Signal Handler, you pressed Ctrl+C! to close the node')
        if not os.environ.get('OS','') == 'Windows_NT': # Windows
            time.sleep(.5)
            os.system('kill %d' % os.getpid())
            sys.exit(0)
        else:
            """Signal handler used to close when user press Ctrl+C"""
            time.sleep(.5)
            import os
            pid = os.getpid()
            print (pid)
            import subprocess as s
            s.Popen('taskkill /F /PID {0}'.format(pid), shell=True)
            sys.exit(0)

    
    def __init__(self, IP = '127.0.0.1' , port = 7000, initial_port_number = 9000):

        """
        Creates a new master object for service discovery

        Parameters
        ----------

        IP : string
            IP value of server

        port : int
            Port used to connect the socket
            
        initial_port_number : int
            Port numbers will be registered starting from the value of this parameter

        """

        # Start master server
        try:
            self.nodes_register = {}
            self.server = nep.server(IP,port)
            self.IP = IP
            self.initial_port_number = initial_port_number

            # Ports configuration
            self.current_port = initial_port_number
            self.topic_register = {}
            self.threads_id = []
            self.brokers = []
            msg_type = "json"         # Message type to listen. "string" or "dict"
            node = nep.node("master_node")                                                                   # Create a new node
            pub_port = 7777
            conf = node.direct(ip = "127.0.0.1", port =  pub_port, mode ="one2many")                # Select the configuration of the publisher
            self.master_pub = node.new_pub("master",msg_type,conf)
            time.sleep(1)
            self.master_pub.publish(self.topic_register)
        except Exception as e:
            print("MASTER ALREADY OPEN")
            sys.exit(0)

    def __onNewTopic(self,node_request):
        """
        Register and return connection parameters for requested topics

        Parameters
        ----------

        node_request : dictionary
            Request parameters of the topic. This parameters are: topic (i.e the topic name), node (i.e. name of the module), socket (e.g. publisher, subscriber, server, client, surveyor, ...) and mode (only for publish and subscriber and can be many2one, one2many and many2many). 
        """

        topic = str(node_request['topic'])

        if node_request["socket"] == "publisher" or node_request["socket"] == "subscriber":
            if node_request["mode"] == "many2many":
                proxy = nep.broker(self.IP, self.current_port+1, self.current_port)
                self.brokers.append(proxy)
                th = thread.start_new_thread (proxy.start, ())
                self.threads_id.append(th)
                time.sleep(.5)
            
            self.topic_register.update({topic:{"port":self.current_port,"socket":node_request["socket"], "mode":node_request["mode"]}})
            msg = {'topic':topic, 'port': self.current_port, 'mode':node_request["mode"],  'ip': self.IP, 'socket':node_request["socket"], "state":"success"}

        elif node_request["socket"] == "surveyor" or node_request["socket"] == "respondent":
            self.topic_register.update({topic:{"port":self.current_port,"socket":node_request["socket"]}})
            msg = {'topic':topic, 'port': self.current_port,  'ip': self.IP, 'socket':node_request["socket"], "state":"success"}
            pass

        elif node_request["socket"] == "client" or node_request["socket"] == "server":
            self.topic_register.update({topic:{"port":self.current_port,"socket":node_request["socket"]}})
            msg = {'topic':topic, 'port': self.current_port,  'ip': self.IP, 'socket':node_request["socket"],"state":"success"}
            pass
        
        self.server.send_info(msg)
        self.topic_register[topic]["nodes"] = {node_request['node']:{'socket':node_request["socket"], 'pid':"n.a."}}
        
        if "pid" in node_request:
            self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':node_request["pid"]}
            #data = {node_request['node']:{'socket':node_request["socket"], 'pid':node_request["pid"]}}
            #data =  {'node':node_request['node'],'socket':node_request["socket"], 'pid':node_request["pid"]}
        #else:
            #self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':"n.a."}
            #data = {node_request['node']:{'socket':node_request["socket"], 'pid':"n.a."}}
            #data =  {'node':node_request['node'],'socket':node_request["socket"], 'pid':"n.a."} 
        #self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':node_request["pid"]}}
        #self.topic_register[topic]["nodes"].append(data)
        self.current_port = self.current_port + 2
        self.master_pub.publish(self.topic_register)
        #print (self.topic_register)

    def __onRegisteredTopic(self,topic, node_request):
        """ 
        Send topic info to the node

        Parameters
        ----------

        topic : string
            Topic name

        node_request : dictionary
            Request parameters of the topic. This parameters are: topic (i.e the topic name), node (i.e. name of the module), socket (e.g. publisher, subscriber, server, client, surveyor, ...) and mode (only for publish and subscriber and can be many2one, one2many and many2many). 
        
        """
        
        try:
            port = self.topic_register[topic]["port"]
            socket_= self.topic_register[topic]["socket"]

            if "mode" in self.topic_register[topic]:
                mode = self.topic_register[topic]["mode"]
                msg = {'topic':topic, 'port': port, 'mode':mode, 'ip': self.IP, 'socket':socket_ , "state":"success"}
            else:
                msg = {'topic':topic, 'port': port, 'ip': self.IP, 'socket':socket_, "state":"success"}

            self.server.send_info(msg)

            if "pid" in node_request:
                self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':node_request["pid"]}

            #if "pid" in node_request:
                #self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':node_request["pid"]}
                #data = {node_request['node']:{'socket':node_request["socket"], 'pid':node_request["pid"]}}
                #data =  {'node':node_request['node'],'socket':node_request["socket"], 'pid':node_request["pid"]}
            #else:
                #self.topic_register[topic]["nodes"][node_request['node']] = {'socket':node_request["socket"], 'pid':"n.a."}

            #if "pid" in node_request:
                #data =  {'node':node_request['node'],'socket':node_request["socket"], 'pid':node_request["pid"], 'port': port, 'ip': self.IP,}
            #else:
                #data =  {'node':node_request['node'],'socket':node_request["socket"], 'pid':"n.a.", 'port': port, 'ip': self.IP,} 

            #self.topic_register[topic]["nodes"].append(data)
            self.master_pub.publish(self.topic_register)
        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)
            print ("Error in topic request")
        
        #print (self.topic_register)

    def __stop(self):
        for key, value in self.topic_register.iteritems():
            nodes = value["nodes"]
            for node in nodes:

                if not os.environ.get('OS','') == 'Windows_NT': # Windows
                    os.system('kill %d' % node["pid"])

                else:
                    """Signal handler used to close when user press Ctrl+C"""
                    import subprocess as s
                    s.Popen('taskkill /F /PID {0}'.format(node["pid"]), shell=True)
    
    def stopNode(self, node):


            print ("Kill")
            print (node)
            if not os.environ.get('OS','') == 'Windows_NT': # Windows
                os.system('kill %d' % node["pid"])

            else:
                import subprocess as s
                s.Popen('taskkill /F /PID {0}'.format(node["pid"]), shell=True)

            if node['kill'] in self.nodes_register:
                self.nodes_register.pop(node['kill'], None)

                for topics in self.topic_register:
                    self.topic_register[topics]["nodes"].pop(node['kill'], None)




    def run(self):
            """ Run master node until Ctrl+C is pressed
            """
            # Enable to kill the node using Ctrl + C
            signal.signal(signal.SIGINT, self.__signal_handler)


            while True:
                time.sleep(.01)
                node_request = self.server.listen_info()
                print ("Request: " + str(node_request))
                if 'node' in node_request:
                    if node_request['node'] in self.nodes_register:
                        current_pid = self.nodes_register[node_request['node']]
                        if node_request['pid'] == current_pid:
                            pass
                        else:
                            try:

                                print ("kill: " + str(current_pid))
                                os.kill(current_pid, signal.SIGTERM)
                                time.sleep(1)


                            except:
                                pass
                    
                    self.nodes_register[node_request['node']] = node_request['pid']

                    topic = str(node_request['topic'])
                    current_pid = node_request['pid']
                    
                    if topic in self.topic_register:
                        self.__onRegisteredTopic(topic, node_request)
                    else:
                        self.__onNewTopic(node_request)
                        

                # Get avaliable topic list
                elif 'topic_list' in node_request: 
                    msg = self.topic_register
                    self.server.send_info(msg)

                elif 'master' in node_request: 
                    action = node_request["master"]
                    if action == "node_kill":
                        self.stopNode(node_request)
                        self.server.send_info({"node":"close_node", "status":"success"})

                    elif action == "kill_all":
                        pass
                        
                    
                    else:
                        self.server.send_info({"node":"action", "status":"failure"})

                else:
                    self.server.send_info({"node":"action", "status":"failure"})


                    #action = node_request["master"]
                    #if action == "stop":
                    #    print ("Master stoping ...")
                    #    time.sleep(1)
                    #    for p in self.brokers:
                    #        p.stop()
                    #    self.server.send_info({"node":"stop", "status":"success"})
                    #    break
                    #elif action == "clean":
                    #    print ("Clean master")
                    #    
                    #    for p in self.brokers:
                    #        self.current_port = self.initial_port_number
                    #        p.stop()


                    #    self.__stop()
                    #    self.topic_register = {}

                        
                    #    self.server.send_info({"node":"clean", "status":"success"})
                    #else:
                    #    self.server.send_info({"node":action, "status":"failure"})
                
                print (self.topic_register)
                print (self.nodes_register)

                        
if __name__ == "__main__":
    import doctest
    doctest.testmod()



# coding = utf-8
#!/usr/bin/env python

# ------------------------------- Broker -----------------------_-----------
# Description: Low-level Broker class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import zmq
import sys
import nep


class broker():
    def __init__(self, IP, PORT_XPUB, PORT_XSUB, TOPIC = ""):
        """
        Creates a XPUB/XSUB broker for many2many publish-subcribe communication

        Parameters
        ----------

        IP : int 
            IP value of the broker

        PORT_XPUB : int 
            XPUB port. Which must be different that PORT_XSUB.

        PORT_XSUB : int 
            XSUB port. Which must be different that PORT_XPUB.

        """
        self.PORT_XPUB = PORT_XPUB
        self.PORT_XSUB = PORT_XSUB
        self.TOPIC =  TOPIC
        self.context = zmq.Context()
        self.frontend = self.context.socket(zmq.XSUB)
        self.frontend.bind("tcp://" + IP + ":" + str(PORT_XSUB))
        self.backend = self.context.socket(zmq.XPUB)
        self.backend.bind("tcp://" + IP + ":" + str(PORT_XPUB))

    def start(self):
        """ Start proxy
        """
        try:
            zmq.proxy(self.frontend, self.backend)
        except:
            if self.TOPIC == "":
                print ("Proxy in (" + str(self.PORT_XSUB) + "," + str(self.PORT_XPUB) + ") stopped")
            else:
                print ("Proxy **" + self.TOPIC + "** in (" + str(self.PORT_XSUB) + "," + str(self.PORT_XPUB) + ") stopped")

        
    def stop(self):
        """ Stop proxy
        """
        self.frontend.close()
        self.backend.close()
        self.context.term()

if __name__ == "__main__":
    import doctest
    doctest.testmod()


# coding = utf-8
#!/usr/bin/env python

# ------------------------------- Server -----------------------------------
# Description: Low-level Server Class
# --------------------------------------------------------------------------
# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.
# Autor: Luis Enrique Coronado Zuniga


import time
import os
import zmq
import simplejson
import sys
import nep
import socket

class server:
    def __init__(self, IP, port, transport = "ZMQ", debug = True):
        """
        Creates a new server object

        Parameters
        ----------

        IP : string
            IP value of server

        port : string
            Port used to connect the socket
            
        transport : string
            Define the transport layer of the server, can be 'ZMQ' or "normal", to use ZeroMQ or TCP/IP python sockets

        debug: bool
            If True some additional information of the socket is shown

        """
        
        self.transport = transport
        self.connected = False

        if transport == "ZMQ":

            # ZMQ sockets
            context = zmq.Context()
                
            try:
                # Define the socket using the "Context"
                self.sock = context.socket(zmq.REP)
                self.sock.bind("tcp://" + IP + ":" + str(port))
                self.connected = True
                if debug:
                    print ("SERVER: ZMQ server in " + IP + ":" + str(port))
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)
                print ('NEP ERROR: Failed in creating socket, already in use')
                self.sock.close()
                context.destroy()
                self.connected = False

        
        elif transport == "normal": 
            # Normal sockets
            try:
                #create an AF_INET, STREAM socket (TCP)
                self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                #self.s.setblocking(0)
                self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            except:
                print ('NEP ERROR: Failed in creating socket, already in use')
                time.sleep(3)
                sys.exit()

            try:
                if IP == "localhost":
                #Same computer
                        IP = socket.gethostbyname( IP )
                        print ("local host in :", IP)
                
            except socket.gaierror:
                #could not resolve
                print ('Hostname could not be resolved. Exiting')
                sys.exit()
            #Connect to remote server
            self.s.bind(('' , int(port)))
            self.s.listen(5)
            print ("SERVER: POSIX server in " + IP + ":" + str(port))
            print ("SERVER: Waiting for connection")
            self.connection, client_address = self.s.accept()
            print ("SERVER: Connection ready")
            self.connected = True

    def __loads(self, s, **kwargs):
        """Load object from JSON bytes (utf-8). See jsonapi.jsonmod.loads for details on kwargs.
        """

        if sys.version_info[0] == 2:
            if str is unicode and isinstance(s, bytes):
                s = s.decode('utf8')
        
        return simplejson.loads(s, **kwargs)

    
    #Status: OK
    def __deserialization(self, info):
        """ JSON deserialization function  (string to dictionary)
            
            Parameters
            ----------
            info : string
                topic + message received by the ZQM socket 

            Returns
            -------
            info : dictionary
                Message as python dictionary
        """
        try:
            msg = self.__loads(info)
            success = True
        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)
            print ("NEP ERROR: msg in not a json value")
            msg = {}
            success = False
        return msg

    def __dumps(self,o, **kwargs):
        """Serialize object to JSON bytes (utf-8).
    
            See jsonapi.jsonmod.dumps for details on kwargs.
        """
        
        if 'separators' not in kwargs:
            kwargs['separators'] = (',', ':')
        
        s = simplejson.dumps(o, **kwargs)
        
        if sys.version_info[0] == 2: #Python 2
            if isinstance(s, unicode):
                s = s.encode('utf8')
        
        return s

    def __serialization(self, message):
        """ Function used to serialize a python dictionary using json.
            
            Parameters
            ----------
            message : dictionary
                Python dictionary to be send
            
            Returns
            -------
            message : string
                Message to be send
        """
        return self.__dumps(message)

    def listen_info(self):
        """ Listen for a json message
                        
            Returns
            -------

            request : dictionary or string
                Message obtained
        """

        if self.transport == "ZMQ" and self.connected:
            #response = self.sock.recv_json()
            #return response
            request = self.sock.recv()
            return self.__deserialization(request)
            
        elif self.connected:
            # Wait for a connection
            try:
                request = self.connection.recv(16)
                return request
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)
                print ("NEP WARNING: client connection lost, waiting for new client")
                self.connection.close()
                self.connection, client_address = self.s.accept()
                request = self.connection.recv(16)
                return request
        else:
            print ("NEP ERROR: socket in use, unable to read")
            if self.transport == "ZMQ":
                return {}
            else:
                return ""

    def send_info(self,response):
        """ Function used to send client response as a python dictionary  (if transport == "ZMQ") or as string (if transport == "normal")
            
            Parameters
            ----------

            response : dictionary or string
                    Python dictionary (ZMQ) or string (Python sockets) to be send

        """
        if self.connected:
            if self.transport == "ZMQ":
                #self.sock.send_json(response)
                if sys.version_info[0] == 2:
                    self.sock.send(self.__serialization(response))
                else:
                    self.sock.send_string(self.__serialization(response))
            else:
                try:
                    self.connection.sendall(response)
                
                except Exception as e:
                    exc_type, exc_obj, exc_tb = sys.exc_info()
                    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                    print(exc_type, fname, exc_tb.tb_lineno)
                    print ("NEP WARNING: client connection lost, waiting for new client")
        else:
            print ("NEP ERROR: socket in use, data not sent")
            time.sleep(2)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



from setuptools import setup

setup(
    name='nep',
    version='0.5.2.0',
    author='Enrique Coronado',
    author_email='enriquecoronadozu@gmail.mx',
    url='http://enriquecoronadozu.github.io',
    description='NEP robot programming framework',
    packages=["nep"],
    install_requires=[
          'pyzmq','simplejson'
      ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development"
    ]
)

